import io
import sys
import os
import json
import re
import argparse
import utils
import requirements
import pipfile
import setup_file
import codecs
from operator import le, lt, gt, ge, eq, ne

try:
    import pkg_resources
except ImportError:
    # try using the version vendored by pip
    try:
        import pip._vendor.pkg_resources as pkg_resources
    except ImportError:
        raise ImportError(
            "Could not import pkg_resources; please install setuptools or pip.")

PYTHON_MARKER_REGEX = re.compile(r'python_version\s*(?P<operator>==|<=|>=|>|<)\s*[\'"](?P<python_version>.+?)[\'"]')
SYSTEM_MARKER_REGEX = re.compile(r'sys_platform\s*==\s*[\'"](.+)[\'"]')

def format_provenance_label(prov_tuple):
    fn, ln1, ln2 = prov_tuple
    if ln1 == ln2:
        return fn + ':' + str(ln1)
    else:
        return fn + ':' + str(ln1) + '-' + str(ln2)

def create_tree_of_packages_dependencies(
        dist_tree,
        top_level_requirements,
        req_file_path,
        allow_missing=False,
        only_provenance=False
    ):
    """Create packages dependencies tree
    :param dict tree: the package tree
    :param set packages_names: set of select packages to be shown in the output.
    :param req_file_path: the path to the dependencies file
                          (e.g. requirements.txt)
    :rtype: dict
    """
    DEPENDENCIES = 'dependencies'
    VERSION = 'version'
    NAME = 'name'
    DIR_VERSION = '0.0.0'
    PACKAGE_FORMAT_VERSION = 'packageFormatVersion'
    LABELS = 'labels'
    PROVENANCE = 'provenance'

    tree = utils.sorted_tree(dist_tree)
    nodes = tree.keys()
    key_tree = dict((k.key, v) for k, v in tree.items())

    lowercase_pkgs_names = [p.name.lower() for p in top_level_requirements]
    tlr_by_key = dict((tlr.name.lower(), tlr) for tlr in top_level_requirements)
    packages_as_dist_obj = [
        p for p in nodes if
            p.key.lower() in lowercase_pkgs_names or
            (p.project_name and p.project_name.lower()) in lowercase_pkgs_names]

    def create_children_recursive(root_package, key_tree, ancestors, all_packages_map):
        root_name = root_package[NAME].lower()
        if root_name not in key_tree:
            msg = 'Required packages missing: ' + root_name
            if allow_missing:
                sys.stderr.write(msg + "\n")
                return
            else:
                sys.exit(msg)

        ancestors = ancestors.copy()
        ancestors.add(root_name)
        children_packages_as_dist = key_tree[root_name]
        for child_dist in children_packages_as_dist:
            child_project_name = child_dist.project_name.lower()
            if child_project_name in ancestors:
                continue

            if DEPENDENCIES not in root_package:
                root_package[DEPENDENCIES] = {}

            if child_project_name in root_package[DEPENDENCIES]:
                continue

            if child_project_name in all_packages_map and child_project_name not in root_package[DEPENDENCIES]:
                root_package[DEPENDENCIES][child_project_name] = 'true'
                continue

            child_package = {
                NAME: child_project_name,
                VERSION: child_dist.installed_version,
            }

            create_children_recursive(child_package, key_tree, ancestors, all_packages_map)
            root_package[DEPENDENCIES][child_project_name] = child_package
            all_packages_map[child_project_name] = 'true'
        return root_package

    def create_dir_as_root():
        name, version = None, None
        if os.path.basename(req_file_path) == 'setup.py':
            with open(req_file_path, "r") as setup_py_file:
                name, version = setup_file.parse_name_and_version(setup_py_file.read())

        dir_as_root = {
            NAME: name or os.path.basename(os.path.dirname(os.path.abspath(req_file_path))),
            VERSION: version or DIR_VERSION,
            DEPENDENCIES: {},
            PACKAGE_FORMAT_VERSION: 'pip:0.0.1'
        }
        return dir_as_root

    def create_package_as_root(package, dir_as_root):
        package_as_root = {
            NAME: package.project_name.lower(),
            # Note: _version is a private field.
            VERSION: package._obj._version,
        }
        return package_as_root
    dir_as_root = create_dir_as_root()
    all_packages_map = {}
    for package in packages_as_dist_obj:
        package_as_root = create_package_as_root(package, dir_as_root)
        if only_provenance:
            package_as_root[LABELS] = {PROVENANCE: format_provenance_label(tlr_by_key[package_as_root[NAME]].provenance)}
            dir_as_root[DEPENDENCIES][package_as_root[NAME]] = package_as_root
        else:
            package_tree = create_children_recursive(package_as_root, key_tree, set([]), all_packages_map)
            dir_as_root[DEPENDENCIES][package_as_root[NAME]] = package_tree
    return dir_as_root

def satisfies_python_requirement(parsed_operator, py_version_str):
    # TODO: use python semver library to compare versions
    operator_func = {
        ">": gt,
        "==": eq,
        "<": lt,
        "<=": le,
        ">=": ge,
        '!=': ne,
    }[parsed_operator]
    system_py_version_tuple = (sys.version_info[0], sys.version_info[1])
    py_version_tuple = tuple(py_version_str.split('.')) # string tuple
    if py_version_tuple[-1] == '*':
        system_py_version_tuple = system_py_version_tuple[0]
        py_version_tuple = int(py_version_tuple[0]) # int tuple
    else:
        py_version_tuple = tuple(int(x) for x in py_version_tuple) # int tuple

    return operator_func(system_py_version_tuple, py_version_tuple)

def get_markers_text(requirement):
    if isinstance(requirement, pipfile.PipfileRequirement):
        return requirement.markers
    return requirement.line

def matches_python_version(requirement):
    """Filter out requirements that should not be installed
    in this Python version.
    See: https://www.python.org/dev/peps/pep-0508/#environment-markers
    """
    markers_text = get_markers_text(requirement)
    if not (markers_text and re.match(".*;.*python_version", markers_text)):
        return True

    cond_text = markers_text.split(";", 1)[1]

    # Gloss over the 'and' case and return true on the first matching python version

    for sub_exp in re.split("\s*(?:and|or)\s*", cond_text):
        match = PYTHON_MARKER_REGEX.search(sub_exp)

        if match:
            match_dict = match.groupdict()

            if len(match_dict) == 2 and satisfies_python_requirement(
                    match_dict['operator'],
                    match_dict['python_version']
                ):
                return True

    return False


def matches_environment(requirement):
    """Filter out requirements that should not be installed
    in this environment. Only sys_platform is inspected right now.
    This should be expanded to include other environment markers.
    See: https://www.python.org/dev/peps/pep-0508/#environment-markers
    """
    sys_platform = sys.platform.lower()
    markers_text = get_markers_text(requirement)
    if markers_text and 'sys_platform' in markers_text:
        match = SYSTEM_MARKER_REGEX.findall(markers_text)
        if len(match) > 0:
            return match[0].lower() == sys_platform
    return True

def is_testable(requirement):
    return requirement.editable == False and requirement.vcs is None

def detect_encoding_by_bom(path):
    with open(path, 'rb') as f:
        raw = f.read(4)    # will read less if the file is smaller
    # BOM_UTF32_LE's start is equal to BOM_UTF16_LE so need to try the former first
    for enc, boms in \
            ('utf-8-sig', (codecs.BOM_UTF8,)), \
            ('utf-32', (codecs.BOM_UTF32_LE, codecs.BOM_UTF32_BE)), \
            ('utf-16', (codecs.BOM_UTF16_LE, codecs.BOM_UTF16_BE)):
        if any(raw.startswith(bom) for bom in boms):
            return enc
    return None

def get_requirements_list(requirements_file_path, dev_deps=False):
    # TODO: refactor recognizing the dependency manager to a single place
    if os.path.basename(requirements_file_path) == 'Pipfile':
        with io.open(requirements_file_path, 'r', encoding='utf-8') as f:
            requirements_data = f.read()
        parsed_reqs = pipfile.parse(requirements_data)
        parsed_packages = parsed_reqs.get('packages', [])
        if parsed_packages is None:
            parsed_packages = []
        req_list = list(parsed_packages)
        if dev_deps:
            req_list.extend(parsed_reqs.get('dev-packages', []))
        if not req_list:
            return []
        else:
            for r in req_list:
                r.provenance = (requirements_file_path, r.provenance[1], r.provenance[2])
    elif os.path.basename(requirements_file_path) == 'setup.py':
        with open(requirements_file_path, 'r') as f:
            setup_py_file_content = f.read()
        requirements_data = setup_file.parse_requirements(setup_py_file_content)
        req_list = list(requirements.parse(requirements_data))

        provenance = setup_file.get_provenance(setup_py_file_content)
        for req in req_list:
            req.provenance = (
                os.path.basename(requirements_file_path),
                provenance,
                provenance
            )
    else:
        # assume this is a requirements.txt formatted file
        # Note: requirements.txt files are unicode and can be in any encoding.
        encoding = detect_encoding_by_bom(requirements_file_path)
        with io.open(requirements_file_path, 'r', encoding=encoding) as f:
            req_list = list(requirements.parse(f))

    req_list = filter(matches_environment, req_list)
    req_list = filter(is_testable, req_list)
    req_list = filter(matches_python_version, req_list)
    req_list = [r for r in req_list if r.name]
    for req in req_list:
        req.name = req.name.lower().replace('_', '-')
    return req_list


def canonicalize_package_name(name):
    # https://packaging.python.org/guides/distributing-packages-using-setuptools/#name
    name = name.lower().replace('-', '.').replace('_', '.')
    name = re.sub(r'\.+', '.', name)
    return name


def create_dependencies_tree_by_req_file_path(requirements_file_path,
                                              allow_missing=False,
                                              dev_deps=False,
                                              only_provenance=False,
                                              allow_empty=False):
    # get all installed packages
    pkgs = list(pkg_resources.working_set)

    # get all installed packages's distribution object
    dist_index = utils.build_dist_index(pkgs)

    # get all installed distributions tree
    dist_tree = utils.construct_tree(dist_index)

    # create a list of dependencies from the dependencies file
    required = get_requirements_list(requirements_file_path, dev_deps=dev_deps)
    if not required and not allow_empty:
        msg = 'No dependencies detected in manifest.'
        sys.exit(msg)
    else:
        installed = [canonicalize_package_name(p) for p in dist_index]
        top_level_requirements = []
        missing_package_names = []
        for r in required:
            if canonicalize_package_name(r.name) not in installed:
                missing_package_names.append(r.name)
            else:
                top_level_requirements.append(r)
        if missing_package_names:
            msg = 'Required packages missing: ' + (', '.join(missing_package_names))
            if allow_missing:
                sys.stderr.write(msg + "\n")
            else:
                sys.exit(msg)

        # build a tree of dependencies
        package_tree = create_tree_of_packages_dependencies(
            dist_tree, top_level_requirements, requirements_file_path, allow_missing, only_provenance)
        print(json.dumps(package_tree))


def main():
    """Builds the dependency tree from the manifest file (Pipfile or requirements.txt) and
    prints it as JSON. The tree nodes are:
    interface DepTree {
        name: string;
        version?: string;
        dependencies?: {[n: string]: DepTree};
        labels: { provenance?: string };
    }
    The `provenance` label only present for the top-level nodes, indicates the position of the dependency
    version in the original file and is in the format "filename:lineNum" or "filename:lineFrom-lineTo",
    where line numbers are 1-based.
    """
    parser = argparse.ArgumentParser()
    parser.add_argument("requirements",
        help="dependencies file path (requirements.txt or Pipfile)")
    parser.add_argument("--allow-missing",
        action="store_true",
        help="don't fail if some packages listed in the dependencies file " +
             "are not installed")
    parser.add_argument("--dev-deps",
        action="store_true",
        help="resolve dev dependencies")
    parser.add_argument("--only-provenance",
        action="store_true",
        help="only return top level deps with provenance information")
    parser.add_argument("--allow-empty",
        action="store_true",
        help="return empty dep tree instead of throwing")
    args = parser.parse_args()

    create_dependencies_tree_by_req_file_path(
        args.requirements,
        allow_missing=args.allow_missing,
        dev_deps=args.dev_deps,
        only_provenance=args.only_provenance,
        allow_empty=args.allow_empty,
    )

if __name__ == '__main__':
    sys.exit(main())



#!/usr/bin/env python3

"""
The main function of some utilities.
"""

import argparse

import pkg_resources
import yaml

import c2cciutils


def main() -> None:
    """
    Run the utilities.
    """
    parser = argparse.ArgumentParser(description="Some utils of c2cciutils.")
    parser.add_argument("--get-config", action="store_true", help="display the current config")
    parser.add_argument("--version", action="store_true", help="display the current version")
    parser.add_argument("--ls-files-mime", help="List all the files with the specified mime type")

    args = parser.parse_args()

    if args.get_config:
        print(yaml.dump(c2cciutils.get_config(), default_flow_style=False, Dumper=yaml.SafeDumper))

    if args.version:
        for pkg in ("c2cciutils", "black", "isort"):
            try:
                print(f"{pkg} {pkg_resources.get_distribution(pkg).version}")
            except pkg_resources.DistributionNotFound:
                print(f"{pkg} missing")

    if args.ls_files_mime:
        for file_name in c2cciutils.get_git_files_mime(args.ls_files_mime):
            print(file_name)


if __name__ == "__main__":
    main()



# MODIFIED - Replace/Update with care

import re

# Copied from pip
# https://github.com/pypa/pip/blob/281eb61b09d87765d7c2b92f6982b3fe76ccb0af/pip/index.py#L947
HASH_ALGORITHMS = set(['sha1', 'sha224', 'sha384', 'sha256', 'sha512', 'md5'])

extras_require_search = re.compile(
    r'(?P<name>.+)\[(?P<extras>[^\]]+)\]').search


def parse_fragment(fragment_string):
    """Takes a fragment string nd returns a dict of the components"""
    fragment_string = fragment_string.lstrip('#')

    try:
        return dict(
            key_value_string.split('==')[0].split('=')
            for key_value_string in fragment_string.split('&')
        )
    except ValueError:
        raise ValueError(
            'Invalid fragment string {fragment_string}'.format(
                fragment_string=fragment_string
            )
        )


def get_hash_info(d):
    """Returns the first matching hashlib name and value from a dict"""
    for key in d.keys():
        if key.lower() in HASH_ALGORITHMS:
            return key, d[key]

    return None, None


def parse_extras_require(egg):
    if egg is not None:
        match = extras_require_search(egg)
        if match is not None:
            name = match.group('name')
            extras = match.group('extras')
            return name, [extra.strip() for extra in extras.split(',')]
    return egg, []



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['c2cciutils',
 'c2cciutils.lib',
 'c2cciutils.node_modules.snyk.pysrc',
 'c2cciutils.node_modules.snyk.pysrc.pytoml',
 'c2cciutils.node_modules.snyk.pysrc.requirements',
 'c2cciutils.scripts',
 'c2cciutils.scripts.k8s']

package_data = \
{'': ['*'],
 'c2cciutils': ['node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/browser.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/constants.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/helpers.js',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.cjs',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/index.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/lib/monorepo.js',
                'node_modules/@pkgr/utils/package.json',
                'node_modules/@toml-tools/lexer/lib/*',
                'node_modules/@toml-tools/lexer/package.json',
                'node_modules/@toml-tools/parser/lib/*',
                'node_modules/@toml-tools/parser/package.json',
                'node_modules/chevrotain/lib/chevrotain.js',
                'node_modules/chevrotain/lib/chevrotain.js',
                'node_modules/chevrotain/lib/chevrotain.min.js',
                'node_modules/chevrotain/lib/chevrotain.min.js',
                'node_modules/chevrotain/lib/src/*',
                'node_modules/chevrotain/lib/src/diagrams/*',
                'node_modules/chevrotain/lib/src/generate/*',
                'node_modules/chevrotain/lib/src/lang/*',
                'node_modules/chevrotain/lib/src/parse/*',
                'node_modules/chevrotain/lib/src/parse/cst/*',
                'node_modules/chevrotain/lib/src/parse/grammar/*',
                'node_modules/chevrotain/lib/src/parse/grammar/gast/*',
                'node_modules/chevrotain/lib/src/parse/parser/*',
                'node_modules/chevrotain/lib/src/parse/parser/traits/*',
                'node_modules/chevrotain/lib/src/scan/*',
                'node_modules/chevrotain/lib/src/text/*',
                'node_modules/chevrotain/lib/src/utils/*',
                'node_modules/chevrotain/package.json',
                'node_modules/cross-spawn/index.js',
                'node_modules/cross-spawn/index.js',
                'node_modules/cross-spawn/lib/*',
                'node_modules/cross-spawn/lib/util/*',
                'node_modules/cross-spawn/package.json',
                'node_modules/cross-spawn/package.json',
                'node_modules/define-lazy-prop/index.js',
                'node_modules/define-lazy-prop/index.js',
                'node_modules/define-lazy-prop/package.json',
                'node_modules/define-lazy-prop/package.json',
                'node_modules/globalyzer/package.json',
                'node_modules/globalyzer/src/index.js',
                'node_modules/globrex/index.js',
                'node_modules/globrex/index.js',
                'node_modules/globrex/package.json',
                'node_modules/globrex/package.json',
                'node_modules/is-docker/cli.js',
                'node_modules/is-docker/cli.js',
                'node_modules/is-docker/cli.js',
                'node_modules/is-docker/index.js',
                'node_modules/is-docker/index.js',
                'node_modules/is-docker/index.js',
                'node_modules/is-docker/package.json',
                'node_modules/is-docker/package.json',
                'node_modules/is-docker/package.json',
                'node_modules/is-extglob/index.js',
                'node_modules/is-extglob/index.js',
                'node_modules/is-extglob/package.json',
                'node_modules/is-extglob/package.json',
                'node_modules/is-glob/index.js',
                'node_modules/is-glob/index.js',
                'node_modules/is-glob/package.json',
                'node_modules/is-glob/package.json',
                'node_modules/is-wsl/index.js',
                'node_modules/is-wsl/index.js',
                'node_modules/is-wsl/package.json',
                'node_modules/is-wsl/package.json',
                'node_modules/isexe/index.js',
                'node_modules/isexe/index.js',
                'node_modules/isexe/index.js',
                'node_modules/isexe/index.js',
                'node_modules/isexe/mode.js',
                'node_modules/isexe/mode.js',
                'node_modules/isexe/mode.js',
                'node_modules/isexe/mode.js',
                'node_modules/isexe/package.json',
                'node_modules/isexe/package.json',
                'node_modules/isexe/package.json',
                'node_modules/isexe/package.json',
                'node_modules/isexe/windows.js',
                'node_modules/isexe/windows.js',
                'node_modules/isexe/windows.js',
                'node_modules/isexe/windows.js',
                'node_modules/mvdan-sh/index.js',
                'node_modules/mvdan-sh/index.js',
                'node_modules/mvdan-sh/package.json',
                'node_modules/mvdan-sh/package.json',
                'node_modules/open/index.js',
                'node_modules/open/index.js',
                'node_modules/open/package.json',
                'node_modules/open/package.json',
                'node_modules/path-key/index.js',
                'node_modules/path-key/index.js',
                'node_modules/path-key/package.json',
                'node_modules/path-key/package.json',
                'node_modules/picocolors/package.json',
                'node_modules/picocolors/package.json',
                'node_modules/picocolors/package.json',
                'node_modules/picocolors/picocolors.browser.js',
                'node_modules/picocolors/picocolors.browser.js',
                'node_modules/picocolors/picocolors.browser.js',
                'node_modules/picocolors/picocolors.js',
                'node_modules/picocolors/picocolors.js',
                'node_modules/picocolors/picocolors.js',
                'node_modules/prettier-plugin-sh/lib/index.cjs',
                'node_modules/prettier-plugin-sh/lib/index.cjs',
                'node_modules/prettier-plugin-sh/lib/index.cjs',
                'node_modules/prettier-plugin-sh/lib/index.cjs',
                'node_modules/prettier-plugin-sh/lib/index.js',
                'node_modules/prettier-plugin-sh/lib/index.js',
                'node_modules/prettier-plugin-sh/lib/index.js',
                'node_modules/prettier-plugin-sh/lib/index.js',
                'node_modules/prettier-plugin-sh/lib/languages.js',
                'node_modules/prettier-plugin-sh/lib/languages.js',
                'node_modules/prettier-plugin-sh/lib/languages.js',
                'node_modules/prettier-plugin-sh/lib/languages.js',
                'node_modules/prettier-plugin-sh/lib/worker.js',
                'node_modules/prettier-plugin-sh/lib/worker.js',
                'node_modules/prettier-plugin-sh/lib/worker.js',
                'node_modules/prettier-plugin-sh/lib/worker.js',
                'node_modules/prettier-plugin-sh/package.json',
                'node_modules/prettier-plugin-toml/lib/*',
                'node_modules/prettier-plugin-toml/package.json',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/bin-prettier.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/cli.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/doc.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/index.js',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/package.json',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-angular.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-babel.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-espree.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-flow.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-glimmer.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-graphql.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-html.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-markdown.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-meriyah.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-postcss.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-typescript.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/parser-yaml.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/standalone.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/prettier/third-party.js',
                'node_modules/regexp-to-ast/lib/*',
                'node_modules/regexp-to-ast/package.json',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/browser.js',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.cjs',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/index.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/processor.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/shim.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/lib/types.js',
                'node_modules/sh-syntax/package.json',
                'node_modules/sh-syntax/vendors/wasm_exec.js',
                'node_modules/shebang-command/index.js',
                'node_modules/shebang-command/index.js',
                'node_modules/shebang-command/package.json',
                'node_modules/shebang-command/package.json',
                'node_modules/shebang-regex/index.js',
                'node_modules/shebang-regex/index.js',
                'node_modules/shebang-regex/package.json',
                'node_modules/shebang-regex/package.json',
                'node_modules/snyk/bin/*',
                'node_modules/snyk/dist/cli/*',
                'node_modules/snyk/dist/cli/commands/*',
                'node_modules/snyk/dist/cli/commands/apps/*',
                'node_modules/snyk/dist/cli/commands/auth/*',
                'node_modules/snyk/dist/cli/commands/fix/*',
                'node_modules/snyk/dist/cli/commands/help/*',
                'node_modules/snyk/dist/cli/commands/monitor/*',
                'node_modules/snyk/dist/cli/commands/protect/*',
                'node_modules/snyk/dist/cli/commands/test/*',
                'node_modules/snyk/dist/cli/commands/test/iac/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/org-settings/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/parsers/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/parsers/hcl-to-json-v2/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/process-results/*',
                'node_modules/snyk/dist/cli/commands/test/iac/local-execution/rules/*',
                'node_modules/snyk/dist/cli/commands/test/iac/v2/*',
                'node_modules/snyk/dist/cli/commands/woof/*',
                'node_modules/snyk/dist/elixirsrc/*',
                'node_modules/snyk/dist/elixirsrc/lib/*',
                'node_modules/snyk/dist/elixirsrc/lib/json/*',
                'node_modules/snyk/dist/elixirsrc/lib/json/lib/*',
                'node_modules/snyk/dist/elixirsrc/lib/json/lib/json/*',
                'node_modules/snyk/dist/elixirsrc/lib/json/lib/json/encoder/*',
                'node_modules/snyk/dist/elixirsrc/lib/json/lib/json/parser/*',
                'node_modules/snyk/dist/elixirsrc/lib/mix/tasks/*',
                'node_modules/snyk/dist/gosrc/*',
                'node_modules/snyk/dist/lib/*',
                'node_modules/snyk/dist/lib/analytics/*',
                'node_modules/snyk/dist/lib/apps/*',
                'node_modules/snyk/dist/lib/apps/create-app/*',
                'node_modules/snyk/dist/lib/config/*',
                'node_modules/snyk/dist/lib/container/*',
                'node_modules/snyk/dist/lib/ecosystems/*',
                'node_modules/snyk/dist/lib/ecosystems/unmanaged/*',
                'node_modules/snyk/dist/lib/errors/*',
                'node_modules/snyk/dist/lib/feature-flags/*',
                'node_modules/snyk/dist/lib/formatters/*',
                'node_modules/snyk/dist/lib/formatters/docker/*',
                'node_modules/snyk/dist/lib/formatters/iac-output/*',
                'node_modules/snyk/dist/lib/formatters/iac-output/text/*',
                'node_modules/snyk/dist/lib/formatters/iac-output/text/failures/*',
                'node_modules/snyk/dist/lib/formatters/iac-output/text/issues-list/*',
                'node_modules/snyk/dist/lib/formatters/test/*',
                'node_modules/snyk/dist/lib/iac/*',
                'node_modules/snyk/dist/lib/iac/assets/*',
                'node_modules/snyk/dist/lib/iac/drift/*',
                'node_modules/snyk/dist/lib/iac/test/v2/*',
                'node_modules/snyk/dist/lib/iac/test/v2/analytics/*',
                'node_modules/snyk/dist/lib/iac/test/v2/local-cache/*',
                'node_modules/snyk/dist/lib/iac/test/v2/local-cache/policy-engine/*',
                'node_modules/snyk/dist/lib/iac/test/v2/local-cache/policy-engine/constants/*',
                'node_modules/snyk/dist/lib/iac/test/v2/local-cache/rules-bundle/*',
                'node_modules/snyk/dist/lib/iac/test/v2/scan/*',
                'node_modules/snyk/dist/lib/module-info/*',
                'node_modules/snyk/dist/lib/monitor/*',
                'node_modules/snyk/dist/lib/plugins/*',
                'node_modules/snyk/dist/lib/plugins/nodejs-plugin/*',
                'node_modules/snyk/dist/lib/plugins/rubygems/*',
                'node_modules/snyk/dist/lib/plugins/rubygems/inspectors/*',
                'node_modules/snyk/dist/lib/plugins/sast/*',
                'node_modules/snyk/dist/lib/plugins/sast/errors/*',
                'node_modules/snyk/dist/lib/plugins/sast/format/*',
                'node_modules/snyk/dist/lib/plugins/sast/utils/*',
                'node_modules/snyk/dist/lib/policy/*',
                'node_modules/snyk/dist/lib/polling/*',
                'node_modules/snyk/dist/lib/project-metadata/*',
                'node_modules/snyk/dist/lib/project-metadata/target-builders/*',
                'node_modules/snyk/dist/lib/request/*',
                'node_modules/snyk/dist/lib/sln/*',
                'node_modules/snyk/dist/lib/snyk-test/*',
                'node_modules/snyk/dist/scala/*',
                'node_modules/snyk/package.json',
                'node_modules/synckit/lib/index.cjs',
                'node_modules/synckit/lib/index.cjs',
                'node_modules/synckit/lib/index.cjs',
                'node_modules/synckit/lib/index.js',
                'node_modules/synckit/lib/index.js',
                'node_modules/synckit/lib/index.js',
                'node_modules/synckit/lib/types.js',
                'node_modules/synckit/lib/types.js',
                'node_modules/synckit/lib/types.js',
                'node_modules/synckit/package.json',
                'node_modules/tiny-glob/index.js',
                'node_modules/tiny-glob/index.js',
                'node_modules/tiny-glob/index.js',
                'node_modules/tiny-glob/package.json',
                'node_modules/tiny-glob/package.json',
                'node_modules/tiny-glob/package.json',
                'node_modules/tiny-glob/sync.js',
                'node_modules/tiny-glob/sync.js',
                'node_modules/tiny-glob/sync.js',
                'node_modules/tslib/modules/*',
                'node_modules/tslib/package.json',
                'node_modules/tslib/package.json',
                'node_modules/tslib/package.json',
                'node_modules/tslib/tslib.es6.js',
                'node_modules/tslib/tslib.es6.js',
                'node_modules/tslib/tslib.es6.js',
                'node_modules/tslib/tslib.js',
                'node_modules/tslib/tslib.js',
                'node_modules/tslib/tslib.js',
                'node_modules/which/package.json',
                'node_modules/which/package.json',
                'node_modules/which/which.js',
                'node_modules/which/which.js']}

install_requires = \
['Markdown>=3.0.0,<4.0.0',
 'PyYAML>=6.0.0,<7.0.0',
 'click>=8.0.0,<9.0.0',
 'debian-inspector>=31.0.0,<32.0.0',
 'defusedxml>=0.0.0,<1.0.0',
 'jsonschema-gentypes>=1.4.0,<1.5.0',
 'jsonschema>=4.0.0,<5.0.0',
 'markdown-table>=2020.0.0,<2021.0.0',
 'node-vm2>=0.0.0,<1.0.0',
 'pyOpenSSL>=24.0.0,<25.0.0',
 'python-magic>=0.0.0,<1.0.0',
 'requests>=2.0.0,<3.0.0',
 'ruamel.yaml>=0.0.0,<1.0.0',
 'toml>=0.0.0,<1.0.0']

extras_require = \
{'checks': ['EditorConfig>=0.0.0,<1.0.0',
            'black==24.3.0',
            'isort==5.12.0',
            'codespell==2.2.6'],
 'pr_checks': ['codespell==2.2.6'],
 'publish': ['google-api-python-client>=2.0.0,<3.0.0',
             'google-auth-httplib2>=0.0.0,<1.0.0',
             'google-auth-oauthlib>=0.0.0,<1.0.0',
             'twine>=4.0.0,<5.0.0',
             'tomlkit>=0.0.0,<1.0.0']}

entry_points = \
{'console_scripts': ['c2cciutils = c2cciutils.scripts.main:main',
                     'c2cciutils-audit = c2cciutils.scripts.audit:main',
                     'c2cciutils-checks = c2cciutils.scripts.checks:main',
                     'c2cciutils-clean = c2cciutils.scripts.clean:main',
                     'c2cciutils-docker-logs = '
                     'c2cciutils.scripts.docker_logs:main',
                     'c2cciutils-docker-versions-gen = '
                     'c2cciutils.scripts.docker_versions_gen:main',
                     'c2cciutils-download-applications = '
                     'c2cciutils.scripts.download_applications:main',
                     'c2cciutils-google-calendar = '
                     'c2cciutils.publish:main_calendar',
                     'c2cciutils-k8s-db = c2cciutils.scripts.k8s.db:main',
                     'c2cciutils-k8s-install = '
                     'c2cciutils.scripts.k8s.install:main',
                     'c2cciutils-k8s-logs = c2cciutils.scripts.k8s.logs:main',
                     'c2cciutils-k8s-wait = c2cciutils.scripts.k8s.wait:main',
                     'c2cciutils-pin-pipenv = '
                     'c2cciutils.scripts.pin_pipenv:main',
                     'c2cciutils-publish = c2cciutils.scripts.publish:main',
                     'c2cciutils-pull-request-checks = '
                     'c2cciutils.scripts.pr_checks:main',
                     'c2cciutils-trigger-image-update = '
                     'c2cciutils.scripts.trigger_image_update:main']}

setup_kwargs = {
    'name': 'c2cciutils',
    'version': '1.5.13',
    'description': 'Common utilities for Camptocamp CI',
    'long_description': '# C2C CI utils\n\nThe goals of C2C CI utils are:\n\n- Have some global checks that\'s didn\'t request any dependency related to the application:\n  this commands return 3 types of results:\n  - Print some useful information:\n    - The version of some packages\n    - The used configuration (with default and autodetect)\n    - The environment variables\n    - The GitHub event file\n  - Check that some configuration where correct:\n    - Git attributes\n    - That the timeout is present in the GitHub workflow files\n    - That the stabilization version (get from the Security.md) are used everywhere it\'s needed\n  - Check the code style:\n    - End of files\n    - Black\n    - Isort\n    - Code spell\n    - Prettier\n  - Snyk tests\n    - Test (never failed)\n    - Code test (never failed, disabled by default)\n    - Iac Test (never failed, disabled by default)\n    - Fix (For information only, disabled by default)\n\nEvery check can be disabled with the following config (the configuration is `ci/config.yaml`):\n\n```yaml\nchecks:\n  <check name>: false\n```\n\nIt make easier to place the following workflows:\n\n- `audit.yaml`: Audit the stabilization branches of the application against vulnerabilities in the python and node dependency\n- `auto-review.yaml`: Auto review the Renovate pull requests\n- `backport.yaml`: Trigger the backports (work with labels)\n- `clean.yaml`: Clean the Docker images related on a deleted feature branch\n- `codeql.yaml`: Run a GitHub CodeQL check\n- `main.yaml`: Main workflow especially with the c2cciutils-checks command\n- `rebuild.yaml`: Daily rebuild of the Docker images on the stabilization branches.\n\nAll the provided commands:\n\n- `c2cciutils`: some generic tools.\n- `c2cciutils-checks`: Run the checks on the code (those checks don\'t need any project dependencies).\n- `c2cciutils-audit`: Do the audit, the main difference with checks is that it can change between runs on the same code.\n- `c2cciutils-publish`: Publish the project.\n- `c2cciutils-clean`: Delete Docker images on Docker Hub after corresponding branch have been deleted.\n- `c2cciutils-google-calendar`: Tool to test the Google credentials for calendar API and refresh them if needed. See `c2cciutils-google-calendar -h` for more information.\n- `c2cciutils-k8s-install`: Install a k3d / k3s cluster, see below.\n- `c2cciutils-k8s-db`: Create a database in the k8s cluster, see below.\n- `c2cciutils-k8s-wait`: Wait that the application started correctly in the cluster, see below.\n- `c2cciutils-k8s-logs`: Display the logs of the application in the k8s cluster, see below.\n- `c2cciutils-pin-pipenv`: Display all the dependencies that\'s in the `Pipenv.lock` but not in the `Pipenv` to be able to pin them.\n- `c2cciutils-docker-logs`: Display the logs of the application in Docker (compose).\n- `c2cciutils-trigger-image-update`: Trigger the ArgoCD repository about image update on the CI (automatically done in the publishing).\n- `c2cciutils-download-applications`: Download the applications with version managed by Renovate, see below.\n- `c2cciutils-docker-versions-gen`: Generate the Docker package versions file (`ci/dpkg-versions.yaml`), see below.\n\n## New project\n\nThe content of `example-project` can be a good base for a new project.\n\n## Secrets\n\nIn the CI we need to have the following secrets::\n\n- `HAS_SECRETS` to be set to \'HAS_SECRETS\', to avoid error errors from external\n  pull requests, already set globally on Camptocamp organization.\n- `GOPASS_CI_GITHUB_TOKEN` and `CI_GPG_PRIVATE_KEY` required to initialize the gopass password store,\n  the secrets exists in the Camptocamp organization but not shared on all project, then you should add\n  your project to the shared list.\n\n## Use locally, in the projects that use c2cciutils\n\nInstall it: `python3 -m pip install --user --requirement ci/requirements.txt`\nRun the checkers: `c2cciutils-checks [--fix] [--stop] [--check CHECK]`\nDry run publish: `GITHUB_REF=... c2cciutils-publish --dry-run ...`\n\n## Configuration\n\nYou can get the current configuration with `c2cciutils --get-config`, the default configuration depends on your project.\nNote that it didn\'t contain the default defined the schema and visible in the [generated documentation](./config.md).\n\nYou can override the configuration with the file `ci/config.yaml`.\n\nAt the base of the configuration you have:\n\n- `version`: Contains some regular expressions to find the versions branches and tags, and to convert them into application versions.\n- `checks`: The checker\'s configuration, see `c2cciutils/checks.py` for more information.\n- `audit`: The audit configuration, see `c2cciutils/audit.py` for more information.\n- `publish`: The publishing configuration, see `c2cciutils/publish.py` for more information.\n\nMany actions can be disabled by setting the corresponding configuration part to `False`.\n\n## SECURITY.md\n\nThe `SECURITY.md` file should contain the security policy of the repository, especially the end of\nsupport dates.\n\nFor compatibility with `c2cciutils` it should contain an array with at least the columns\n`Version` and `Supported Until`. The `Version` column will contain the concerned version.\nThe `Supported Until` will contain the date of end of support `dd/mm/yyyy`.\nIt can also contain the following sentences:\n\n- `Unsupported`: no longer supported => no audit, no rebuild.\n- `Best effort`: the support is ended, it is still rebuilt and audited, but this can be stopped without any notice.\n- `To be defined`: not yet released or the date will be set related of another project release date (like for GeoMapFish).\n\nSee also [GitHub Documentation](https://docs.github.com/en/github/managing-security-vulnerabilities/adding-a-security-policy-to-your-repository)\n\n## IDE\n\nThe IDE should be configured as:\n\n- using `black` and `isort` without any arguments,\n- using the `editorconfig` configuration.\n\n### VScode\n\n- Recommend extensions to work well with c2cciutils:\n  - [Prettier](https://marketplace.visualstudio.com/items?itemName=esbenp.prettier-vscode) And use EditorConfig\n  - [shell-format](https://marketplace.visualstudio.com/items?itemName=foxundermoon.shell-format) With the configuration\n    `"shellformat.flag": "-bn"`.\n  - [Better TOML](https://marketplace.visualstudio.com/items?itemName=bodil.prettier-toml)\n- Other recommend extensions:\n  - [hadolint](https://marketplace.visualstudio.com/items?itemName=exiasr.hadolint)\n  - [Code Spell Checker](https://marketplace.visualstudio.com/items?itemName=streetsidesoftware.code-spell-checker)\n\nSelect a formatter:\n\n- `CTRL+MAJ+P`\n- Format document With...\n- Configure Default Formatter...\n- Select the formatter\n\n## Publishing\n\n### To pypi\n\nThe config is like this:\n\n```yaml\nversions:\n  # List of kinds of versions you want to publish, that can be:\n  # rebuild (specified with --type),\n  # version_tag, version_branch, feature_branch, feature_tag (for pull request)\n```\n\nIt we have a `setup.py` file, we will be in legacy mode:\nWhen publishing, the version computed from arguments or `GITHUB_REF` is put in environment variable `VERSION`, thus you should use it in `setup.py`, example:\n\n```python\nVERSION = os.environ.get("VERSION", "1.0.0")\n```\n\nAlso we consider that we use `poetry` with [poetry-dynamic-versioning](https://pypi.org/project/poetry-dynamic-versioning/) to manage the version, and [poetry-plugin-tweak-dependencies-version](https://pypi.org/project/poetry-plugin-tweak-dependencies-version/) to manage the dependencies versions.\n\nExample of configuration:\n\n```toml\n[tool.poetry-dynamic-versioning]\nenable = true\nvcs = "git"\npattern = "^(?P<base>\\\\d+(\\\\.\\\\d+)*)"\nformat-jinja = """\n{%- if env.get("VERSION_TYPE") == "version_branch" -%}\n{{serialize_pep440(bump_version(base, 1 if env.get("IS_MASTER") == "TRUE" else 2), dev=distance)}}\n{%- elif distance == 0 -%}\n{{serialize_pep440(base)}}\n{%- else -%}\n{{serialize_pep440(bump_version(base), dev=distance)}}\n{%- endif -%}\n"""\n\n```\n\nNote that we can access to the environment variables `VERSION`,`VERSION_TYPE` and `IS_MASTER`.\n\nThen by default:\n\n- Tag with `1.2.3` => release `1.2.3`\n- Commit on feature branch just do a validation\n- Commit on `master` branch after the tag 1.3.0 => release `1.4.0.dev1`\n- Commit on `1.3` branch after the tag 1.3.0 => release `1.3.1.dev1`\n\nTo make it working in the `Dockerfile` you should have in the `poetry` stage:\n\n```Dockerfile\nENV POETRY_DYNAMIC_VERSIONING_BYPASS=dev\nRUN poetry export --extras=checks --extras=publish --extras=audit --output=requirements.txt \\\n    && poetry export --with=dev --output=requirements-dev.txt\n```\n\nAnd in the `run` stage\n\n```Dockerfile\nARG VERSION=dev\nRUN --mount=type=cache,target=/root/.cache \\\n    POETRY_DYNAMIC_VERSIONING_BYPASS=${VERSION} python3 -m pip install --disable-pip-version-check --no-deps --editable=.\n```\n\nAnd in the `Makefile`:\n\n```Makefile\nVERSION = $(strip $(shell poetry version --short))\n\n.PHONY: build\nbuild: ## Build the Docker images\n    docker build --build-arg=VERSION=$(VERSION) --tag=$(GITHUB_REPOSITORY) .\n```\n\n### To Docker registry\n\nThe config is like this:\n\n```yaml\nlatest: True\nimages:\n  - name: # The base name of the image we want to publish\nrepository:\n  <internal_name>:\n    \'server\': # The fqdn name of the server if not Docker hub\n    \'version\':# List of kinds of versions you want to publish, that can be: rebuild (specified using --type),\n      # version_tag, version_branch, feature_branch, feature_tag (for pull request)\n    \'tags\':# List of tags we want to publish interpreted with `template(version=version)`\n      # e.g. if you use `{version}-lite` when you publish the version `1.2.3` the source tag\n      # (that should be built by the application build) is `latest-lite`, and it will be published\n      # with the tag `1.2.3-lite`.\n    \'group\':# If your images are published by different jobs you can separate them in different groups\n      # and publish them with `c2cciutils-publish --group=<group>`\n```\n\nBy default, the last line of the `SECURITY.md` file will be published (`docker`) with the tag\n`latest`. Set `latest` to `False` to disable it.\n\nWith the `c2cciutils-clean` the images on Docker hub for `feature_branch` will be removed on branch removing.\n\n## Download applications\n\nIn case some executables or applications from GitHub releases or any other URLs are required on the CI host\nand are not handled by any dependency manager, we provide a set of tools to install them and manage upgrades\nthrough Renovate.\n\nCreate an application file (e.-g. `applications.yaml`) with:\n\n```yaml\n# yaml-language-server: $schema=https://raw.githubusercontent.com/camptocamp/c2cciutils/master/c2cciutils/schema-applications.json\n\n# Application from GitHub release\n<organization>/<project>:\n  get-file-name: <file name present in the release>\n  to-file-name: <The file name you want to create in ~/.local/bin>\n  finish-command: # The command you want to run after the file is downloaded\n    - - chmod # To be executable (usually required)\n      - +x\n      - <to-file-name>\n    - - <to-file-name> # Print the version of the application\n      - --version\n# Application from GitHub release in a tar file (or tar.gz)\n<organization>/<project>:\n  get-file-name: <file name present in the release>\n  type: tar\n  tar-file-name: <The file name available in the tar file>\n  to-file-name: <The file name you want to create in ~/.local/bin>\n  finish-command: [...] # The command you want to run after the file is downloaded\n# Application from an URL\n<application reference name>:\n  url-pattern: <The URL used to download the application>\n  to-file-name: <The file name you want to create in ~/.local/bin>\n  finish-command: [...] # The command you want to run after the file is downloaded\n```\n\nIn the attributes `url-pattern`, `get-file-name` you can use the following variables:\n\n- `{version}`: The version of the application present in the version file.\n- `{version_quote}`: The URL encoded version.\n- `{short_version}`: The version without the `v` prefix.\n\nThe `applications-versions.yaml` file is a map of applications and their versions.\n\nAdd in your Renovate configuration:\n\n```json5\n\n  regexManagers: [\n    {\n      fileMatch: [\'^applications-versions.yaml$\'],\n      matchStrings: [\n        \'(?<depName>[^\\\\s]+): (?<currentValue>[^\\\\s]+) # (?<datasource>[^\\\\s]+)\',\n      ],\n    },\n  ],\n```\n\nNow you need to call `c2cciutils-download-applications --applications-file=applications.yaml --versions-file=applications-version.yaml`\nto install required applications on CI host before using them (an already installed application is installed only if needed).\n\n## Use Renovate to trigger a new build instead of the legacy rebuild\n\nRun the command `c2cciutils-docker-versions-gen camptocamp/image[:tag]` to generate a file that is a kind of package lock of the Debian packages in the file `ci/dpkg-versions.yaml`.\n\nAdd in your renovate configuration:\n\n```javascript\n  regexManagers: [\n    {\n      fileMatch: [\'^ci/dpkg-versions.yaml$\'],\n      matchStrings: [" *(?<depName>[^\'\\\\s]+): \'?(?<currentValue>[^\'\\\\s/]*[0-9][^\'\\\\s/]*)\'?"],\n      datasourceTemplate: \'repology\',\n      versioningTemplate: \'loose\',\n    },\n  ],\n```\n\nWhen a new version of a Debian package will be available:\n\n- Renovate will automatically open a pull request to update the file `ci/dpkg-versions.yaml`.\n- And the continuous integration will build a new fresh Docker image with latest versions of all Debian packages.\n\n## Kubernetes\n\nC2cciutils provide some commands for Kubernetes.\n\nYou can define a workflow like that:\n\n```yaml\n- name: Install k3s/k3d (Kubernetes cluster)\n  run: c2cciutils-k8s-install\n\n- name: Create a database to do the tests\n  run: c2cciutils-k8s-db --script=<my_script>.sql\n\n- name: Install the application in the Kubernetes cluster\n  run: kubectl apply -f <my_application>.yaml\n\n- name: Wait that the application is ready\n  run: c2cciutils-k8s-wait\n- name: Print the application status and logs\n  run: c2cciutils-k8s-logs\n  if: always()\n\n- name: Uninstall the application\n  run: kubectl delete -f <my_application>.yaml || true\n\n- name: Cleanup the database\n  run: c2cciutils-k8s-db --cleanup\n```\n\n`c2cciutils-k8s-install` can be configured in the `ci/config.yaml` file, in section `k8s/k3d/install-commands`, default is:\n\n```yaml\n- - k3d\n    cluster\n    create\n    test-cluster\n    --no-lb\n    --no-rollback\n```\n\nSee also: [K3d cluster create documentation](https://k3d.io/v4.4.8/usage/commands/k3d_cluster_create/).\n\n`c2cciutils-k8s-db` can be configured in the `ci/config.yaml` file, in section `k8s/db/chart-options`, default is:\n\n```yaml\npersistence.enabled: \'false\'\ntls.enabled: \'true\'\ntls.autoGenerated: \'true\'\npostgresqlPassword: mySuperTestingPassword\nvolumePermissions.enabled: \'true\'\n```\n\nSee also: [Parameters documentations](https://github.com/bitnami/charts/tree/master/bitnami/postgresql#parameters).\n',
    'author': 'Camptocamp',
    'author_email': 'info@camptocamp.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/camptocamp/c2cciutils',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import argparse

import yaml

import c2cciutils.lib.docker


def main() -> None:
    """Dump the actual versions of packages in image to file ci/dpkg-versions.yaml."""

    argparser = argparse.ArgumentParser(
        description="Dump the actual versions of packages in image to file ci/dpkg-versions.yaml."
    )
    argparser.add_argument("--distribution", help="The default distribution code to be used")
    argparser.add_argument("--release", help="The default release version to be used")
    argparser.add_argument("images", help="The image to check", nargs="+")
    args = argparser.parse_args()

    versions_config, _ = c2cciutils.lib.docker.get_versions_config()
    for image in args.images:

        _, versions_image = c2cciutils.lib.docker.get_dpkg_packages_versions(
            image,
            default_distribution=args.distribution,
            default_release=args.release,
        )
        versions_config[image] = {k: str(v) for k, v in versions_image.items()}

    with open("ci/dpkg-versions.yaml", "w", encoding="utf-8") as versions_file:
        versions_file.write("# See repository list: https://repology.org/repositories/statistics\n\n")
        versions_file.write(yaml.dump(versions_config, Dumper=yaml.SafeDumper, default_flow_style=False))


if __name__ == "__main__":
    main()



import string, re, sys, datetime
from .core import TomlError

if sys.version_info[0] == 2:
    _chr = unichr
else:
    _chr = chr

def load(fin, translate=lambda t, x, v, pos: v, object_pairs_hook=dict):
    return loads(fin.read(), translate=translate, object_pairs_hook=object_pairs_hook, filename=getattr(fin, 'name', repr(fin)))

def loads(s, filename='<string>', translate=lambda t, x, v, pos: v, object_pairs_hook=dict):
    if isinstance(s, bytes):
        s = s.decode('utf-8')

    s = s.replace('\r\n', '\n')

    root = object_pairs_hook()
    tables = object_pairs_hook()
    scope = root

    src = _Source(s, filename=filename)
    ast = _p_toml(src, object_pairs_hook=object_pairs_hook)

    def error(msg):
        raise TomlError(msg, pos[0], pos[1], filename)

    def process_value(v, object_pairs_hook):
        kind, text, value, pos = v
        if kind == 'str' and value.startswith('\n'):
            value = value[1:]
        if kind == 'array':
            if value and any(k != value[0][0] for k, t, v, p in value[1:]):
                error('array-type-mismatch')
            value = [process_value(item, object_pairs_hook=object_pairs_hook) for item in value]
        elif kind == 'table':
            value = object_pairs_hook([(k, process_value(value[k], object_pairs_hook=object_pairs_hook)) for k in value])
        return translate(kind, text, value, pos)

    for kind, value, pos in ast:
        if kind == 'kv':
            k, v = value
            if k in scope:
                error('duplicate_keys. Key "{0}" was used more than once.'.format(k))
            scope[k] = process_value(v, object_pairs_hook=object_pairs_hook)
        else:
            is_table_array = (kind == 'table_array')
            cur = tables
            for name in value[:-1]:
                if isinstance(cur.get(name), list):
                    d, cur = cur[name][-1]
                else:
                    d, cur = cur.setdefault(name, (None, object_pairs_hook()))

            scope = object_pairs_hook()
            name = value[-1]
            if name not in cur:
                if is_table_array:
                    cur[name] = [(scope, object_pairs_hook())]
                else:
                    cur[name] = (scope, object_pairs_hook())
            elif isinstance(cur[name], list):
                if not is_table_array:
                    error('table_type_mismatch')
                cur[name].append((scope, object_pairs_hook()))
            else:
                if is_table_array:
                    error('table_type_mismatch')
                old_scope, next_table = cur[name]
                if old_scope is not None:
                    error('duplicate_tables')
                cur[name] = (scope, next_table)

    def merge_tables(scope, tables):
        if scope is None:
            scope = object_pairs_hook()
        for k in tables:
            if k in scope:
                error('key_table_conflict')
            v = tables[k]
            if isinstance(v, list):
                scope[k] = [merge_tables(sc, tbl) for sc, tbl in v]
            else:
                scope[k] = merge_tables(v[0], v[1])
        return scope

    return merge_tables(root, tables)

class _Source:
    def __init__(self, s, filename=None):
        self.s = s
        self._pos = (1, 1)
        self._last = None
        self._filename = filename
        self.backtrack_stack = []

    def last(self):
        return self._last

    def pos(self):
        return self._pos

    def fail(self):
        return self._expect(None)

    def consume_dot(self):
        if self.s:
            self._last = self.s[0]
            self.s = self[1:]
            self._advance(self._last)
            return self._last
        return None

    def expect_dot(self):
        return self._expect(self.consume_dot())

    def consume_eof(self):
        if not self.s:
            self._last = ''
            return True
        return False

    def expect_eof(self):
        return self._expect(self.consume_eof())

    def consume(self, s):
        if self.s.startswith(s):
            self.s = self.s[len(s):]
            self._last = s
            self._advance(s)
            return True
        return False

    def expect(self, s):
        return self._expect(self.consume(s))

    def consume_re(self, re):
        m = re.match(self.s)
        if m:
            self.s = self.s[len(m.group(0)):]
            self._last = m
            self._advance(m.group(0))
            return m
        return None

    def expect_re(self, re):
        return self._expect(self.consume_re(re))

    def __enter__(self):
        self.backtrack_stack.append((self.s, self._pos))

    def __exit__(self, type, value, traceback):
        if type is None:
            self.backtrack_stack.pop()
        else:
            self.s, self._pos = self.backtrack_stack.pop()
        return type == TomlError

    def commit(self):
        self.backtrack_stack[-1] = (self.s, self._pos)

    def _expect(self, r):
        if not r:
            raise TomlError('msg', self._pos[0], self._pos[1], self._filename)
        return r

    def _advance(self, s):
        suffix_pos = s.rfind('\n')
        if suffix_pos == -1:
            self._pos = (self._pos[0], self._pos[1] + len(s))
        else:
            self._pos = (self._pos[0] + s.count('\n'), len(s) - suffix_pos)

_ews_re = re.compile(r'(?:[ \t]|#[^\n]*\n|#[^\n]*\Z|\n)*')
def _p_ews(s):
    s.expect_re(_ews_re)

_ws_re = re.compile(r'[ \t]*')
def _p_ws(s):
    s.expect_re(_ws_re)

_escapes = { 'b': '\b', 'n': '\n', 'r': '\r', 't': '\t', '"': '"', '\'': '\'',
    '\\': '\\', '/': '/', 'f': '\f' }

_basicstr_re = re.compile(r'[^"\\\000-\037]*')
_short_uni_re = re.compile(r'u([0-9a-fA-F]{4})')
_long_uni_re = re.compile(r'U([0-9a-fA-F]{8})')
_escapes_re = re.compile('[bnrt"\'\\\\/f]')
_newline_esc_re = re.compile('\n[ \t\n]*')
def _p_basicstr_content(s, content=_basicstr_re):
    res = []
    while True:
        res.append(s.expect_re(content).group(0))
        if not s.consume('\\'):
            break
        if s.consume_re(_newline_esc_re):
            pass
        elif s.consume_re(_short_uni_re) or s.consume_re(_long_uni_re):
            res.append(_chr(int(s.last().group(1), 16)))
        else:
            s.expect_re(_escapes_re)
            res.append(_escapes[s.last().group(0)])
    return ''.join(res)

_key_re = re.compile(r'[0-9a-zA-Z-_]+')
def _p_key(s):
    with s:
        s.expect('"')
        r = _p_basicstr_content(s, _basicstr_re)
        s.expect('"')
        return r
    if s.consume('\''):
        if s.consume('\'\''):
            r = s.expect_re(_litstr_ml_re).group(0)
            s.expect('\'\'\'')
        else:
            r = s.expect_re(_litstr_re).group(0)
            s.expect('\'')
        return r
    return s.expect_re(_key_re).group(0)

_float_re = re.compile(r'[+-]?(?:0|[1-9](?:_?\d)*)(?:\.\d(?:_?\d)*)?(?:[eE][+-]?(?:\d(?:_?\d)*))?')
_datetime_re = re.compile(r'(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2})(\.\d+)?(?:Z|([+-]\d{2}):(\d{2}))')

_basicstr_ml_re = re.compile(r'(?:(?:|"|"")[^"\\\000-\011\013-\037])*')
_litstr_re = re.compile(r"[^'\000-\037]*")
_litstr_ml_re = re.compile(r"(?:(?:|'|'')(?:[^'\000-\011\013-\037]))*")
def _p_value(s, object_pairs_hook):
    pos = s.pos()

    if s.consume('true'):
        return 'bool', s.last(), True, pos
    if s.consume('false'):
        return 'bool', s.last(), False, pos

    if s.consume('"'):
        if s.consume('""'):
            r = _p_basicstr_content(s, _basicstr_ml_re)
            s.expect('"""')
        else:
            r = _p_basicstr_content(s, _basicstr_re)
            s.expect('"')
        return 'str', r, r, pos

    if s.consume('\''):
        if s.consume('\'\''):
            r = s.expect_re(_litstr_ml_re).group(0)
            s.expect('\'\'\'')
        else:
            r = s.expect_re(_litstr_re).group(0)
            s.expect('\'')
        return 'str', r, r, pos

    if s.consume_re(_datetime_re):
        m = s.last()
        s0 = m.group(0)
        r = map(int, m.groups()[:6])
        if m.group(7):
            micro = float(m.group(7))
        else:
            micro = 0

        if m.group(8):
            g = int(m.group(8), 10) * 60 + int(m.group(9), 10)
            tz = _TimeZone(datetime.timedelta(0, g * 60))
        else:
            tz = _TimeZone(datetime.timedelta(0, 0))

        y, m, d, H, M, S = r
        dt = datetime.datetime(y, m, d, H, M, S, int(micro * 1000000), tz)
        return 'datetime', s0, dt, pos

    if s.consume_re(_float_re):
        m = s.last().group(0)
        r = m.replace('_','')
        if '.' in m or 'e' in m or 'E' in m:
            return 'float', m, float(r), pos
        else:
            return 'int', m, int(r, 10), pos

    if s.consume('['):
        items = []
        with s:
            while True:
                _p_ews(s)
                items.append(_p_value(s, object_pairs_hook=object_pairs_hook))
                s.commit()
                _p_ews(s)
                s.expect(',')
                s.commit()
        _p_ews(s)
        s.expect(']')
        return 'array', None, items, pos

    if s.consume('{'):
        _p_ws(s)
        items = object_pairs_hook()
        if not s.consume('}'):
            k = _p_key(s)
            _p_ws(s)
            s.expect('=')
            _p_ws(s)
            items[k] = _p_value(s, object_pairs_hook=object_pairs_hook)
            _p_ws(s)
            while s.consume(','):
                _p_ws(s)
                k = _p_key(s)
                _p_ws(s)
                s.expect('=')
                _p_ws(s)
                items[k] = _p_value(s, object_pairs_hook=object_pairs_hook)
                _p_ws(s)
            s.expect('}')
        return 'table', None, items, pos

    s.fail()

def _p_stmt(s, object_pairs_hook):
    pos = s.pos()
    if s.consume(   '['):
        is_array = s.consume('[')
        _p_ws(s)
        keys = [_p_key(s)]
        _p_ws(s)
        while s.consume('.'):
            _p_ws(s)
            keys.append(_p_key(s))
            _p_ws(s)
        s.expect(']')
        if is_array:
            s.expect(']')
        return 'table_array' if is_array else 'table', keys, pos

    key = _p_key(s)
    _p_ws(s)
    s.expect('=')
    _p_ws(s)
    value = _p_value(s, object_pairs_hook=object_pairs_hook)
    return 'kv', (key, value), pos

_stmtsep_re = re.compile(r'(?:[ \t]*(?:#[^\n]*)?\n)+[ \t]*')
def _p_toml(s, object_pairs_hook):
    stmts = []
    _p_ews(s)
    with s:
        stmts.append(_p_stmt(s, object_pairs_hook=object_pairs_hook))
        while True:
            s.commit()
            s.expect_re(_stmtsep_re)
            stmts.append(_p_stmt(s, object_pairs_hook=object_pairs_hook))
    _p_ews(s)
    s.expect_eof()
    return stmts

class _TimeZone(datetime.tzinfo):
    def __init__(self, offset):
        self._offset = offset

    def utcoffset(self, dt):
        return self._offset

    def dst(self, dt):
        return None

    def tzname(self, dt):
        m = self._offset.total_seconds() // 60
        if m < 0:
            res = '-'
            m = -m
        else:
            res = '+'
        h = m // 60
        m = m - h * 60
        return '{}{:.02}{:.02}'.format(res, h, m)



# run with:
# cd pysrc; python3 test_pip_resolve.py; cd ..

from pip_resolve import satisfies_python_requirement, \
                        matches_python_version, \
                        matches_environment, \
                        canonicalize_package_name
from collections import namedtuple

import unittest

try:
    from mock import patch
except:
    from unittest.mock import patch

class TestStringMethods(unittest.TestCase):

    def test_canonicalize_package_name(self):
        # https://packaging.python.org/guides/distributing-packages-using-setuptools/#name
        self.assertEqual(canonicalize_package_name("Cool-Stuff"), "cool.stuff")
        self.assertEqual(canonicalize_package_name("Cool--.--Stuff"), "cool.stuff")
        self.assertEqual(canonicalize_package_name("Cool--__.__--Stuff"), "cool.stuff")

        self.assertEqual(canonicalize_package_name("cool.stuff"), "cool.stuff")
        self.assertEqual(canonicalize_package_name("COOL_STUFF"), "cool.stuff")
        self.assertEqual(canonicalize_package_name("CoOl__-.-__sTuFF"), "cool.stuff")


    def test_satisfies_python_requirement(self):

        with patch('pip_resolve.sys') as mock_sys:
            mock_sys.version_info = (2, 5)
            self.assertTrue(satisfies_python_requirement('>', '2.4'))

            mock_sys.version_info = (2, 3)
            self.assertTrue(satisfies_python_requirement('==', '2.3'))
            self.assertTrue(satisfies_python_requirement('<=', '2.3'))
            self.assertFalse(satisfies_python_requirement('<', '2.3'))

            mock_sys.version_info = (3, 5)
            self.assertTrue(satisfies_python_requirement('>', '3.1'))

            mock_sys.version_info = (2, 8)
            self.assertFalse(satisfies_python_requirement('>', '3.1'))

            mock_sys.version_info = (2, 6)
            self.assertTrue(satisfies_python_requirement('==', '2.*'))

            mock_sys.version_info = (3, 6)
            self.assertTrue(satisfies_python_requirement('==', '3.*'))


    def test_matches_python_version(self):

        req = namedtuple('requirement', ['line'])

        with patch('pip_resolve.sys') as mock_sys:

            mock_sys.version_info = (2, 5)
            req.line = "futures==3.2.0; python_version == '2.6'"
            self.assertFalse(matches_python_version(req))

            mock_sys.version_info = (2, 6)
            req.line = "futures==3.2.0; python_version == '2.6'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 5)
            req.line = "futures==3.2.0; python_version <= '2.6'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 5)
            req.line = 'futures==3.2.0; python_version <= "2.6"'
            self.assertTrue(matches_python_version(req))

            # BUG: python_version is always expected on the left side
            # mock_sys.version_info = (2, 5)
            # req.line = 'futures==3.2.0; "2.6" >= python_version'
            # self.assertTrue(matches_python_version(req))

            # BUG: Double quotes are supported but allow illegal statements
            mock_sys.version_info = (2, 5)
            req.line = '''futures==3.2.0; python_version <= '2.6"'''
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 6)
            req.line = "futures==3.2.0; python_version == '2.6' or python_version == '2.7'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 7)
            req.line = "futures==3.2.0 ; python_version == '2.6' or python_version == '2.7'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 7)
            req.line = "futures==3.2.0 ; python_version == '2.5' or python_version == '2.6'" \
                " or python_version == '2.7'"
            self.assertTrue(matches_python_version(req))

            # BUG: Comments are not supported
            #mock_sys.version_info = (2, 7)
            #req.line = "futures==3.2.0 ; python_version == '2.6' # or python_version == '2.7'"
            #self.assertFalse(matches_python_version(req))

            # BUG: The 'and' case doesn't really make sesne but should be handled correctly
            mock_sys.version_info = (2, 7)
            req.line = "futures==3.2.0 ; python_version == '2.6' and python_version == '2.7'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 6)
            req.line = "futures==3.2.0; python_version == '2.6' and sys_platform == 'linux2'"
            self.assertTrue(matches_python_version(req))

            mock_sys.version_info = (2, 7)
            req.line = "futures==3.2.0; python_version == '2.6' and sys_platform == 'linux2'"
            self.assertFalse(matches_python_version(req))


    def test_matches_environment(self):

        req = namedtuple('requirement', ['line'])

        with patch('pip_resolve.sys') as mock_sys:

            mock_sys.platform = "LInux2"
            req.line = "futures==3.2.0; sys_platform == 'linux2'"
            self.assertTrue(matches_environment(req))

            # BUG: sys_platform is always expected on the left side
            # mock_sys.platform = "win2000"
            # req.line = "futures==3.2.0; 'linux2' == sys_platform"
            # self.assertFalse(matches_environment(req))

            mock_sys.platform = "linux2"
            req.line = 'futures==3.2.0; sys_platform == "linux2"'
            self.assertTrue(matches_environment(req))

            mock_sys.platform = "win2000"
            req.line = "futures==3.2.0; sys_platform == 'linux2'"
            self.assertFalse    (matches_environment(req))

            # BUG: Only == operator is supported in the moment
            # mock_sys.platform = "linux2"
            # req.line = "futures==3.2.0; sys_platform != 'linux2'"
            # self.assertTrue(matches_environment(req))

            # BUG: Expressions containing logical operators are not supported
            # mock_sys.platform = "win2000"
            # req.line = "futures==3.2.0; python_version == '2.6' and sys_platform == 'linux2'"
            # self.assertTrue(matches_environment(req))



if __name__ == '__main__':
    unittest.main()



"""
The checking functions.
"""

import glob
import os
import re
import subprocess  # nosec
import sys
from argparse import Namespace
from io import StringIO
from typing import Any, Dict

import ruamel.yaml
import yaml
from ruamel.yaml.comments import CommentedMap

import c2cciutils
import c2cciutils.configuration
import c2cciutils.prettier
import c2cciutils.security


def print_config(
    config: None,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Print the configuration.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, args

    yaml_ = ruamel.yaml.YAML()
    yaml_.default_flow_style = False
    out = StringIO()
    yaml_.dump(full_config, out)
    print(out.getvalue())
    return True


def print_environment_variables(
    config: None, full_config: c2cciutils.configuration.Configuration, args: Namespace
) -> bool:
    """
    Print the environment variables.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config, args

    for name, value in sorted(os.environ.items()):
        print(f"{name}: {value}")
    return True


def print_github_event(
    config: None, full_config: c2cciutils.configuration.Configuration, args: Namespace
) -> bool:
    """
    Print the GitHub event.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config, args

    if "GITHUB_EVENT_PATH" in os.environ:
        with open(os.environ["GITHUB_EVENT_PATH"], encoding="utf-8") as event:
            print(event.read())
    return True


def _check_properties(
    check: str, file: str, path: str, properties: CommentedMap, reference: Dict[str, Any]
) -> bool:
    if path:
        path += "."

    success = True

    for key, value in reference.items():
        if key not in properties:
            c2cciutils.error(
                check,
                f"The property '{path}{key}' should be defined",
                file,
                properties.lc.line + 1,
                properties.lc.col + 1,
            )
            success = False
        if isinstance(value, dict):
            if not isinstance(properties[key], dict):
                c2cciutils.error(
                    check,
                    f"The property '{path}{key}' should be a dictionary",
                    file,
                    properties.lc.line + 1,
                    properties.lc.col + 1,
                )
                success = False
            else:
                success &= _check_properties(check, file, path + key, properties[key], value)
        else:
            if properties.get(key) != value:
                c2cciutils.error(
                    check,
                    f"The property '{path}{key}' should have the value, '{value}', "
                    f"but is '{properties.get(key)}'",
                    file,
                    properties.lc.line + 1,
                    properties.lc.col + 1,
                )
                success = False
    return success


def gitattribute(
    config: None,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Check that we don't have any error with the gitattributes.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config, args

    try:
        sys.stdout.flush()
        sys.stderr.flush()
        git_ref = (
            subprocess.check_output(["git", "--no-pager", "log", "--oneline"])
            .decode()
            .strip()
            .split("\n", maxsplit=1)[-1]
            .split(" ")[0]
        )
        subprocess.check_call(["git", "--no-pager", "diff", "--no-renames", "--check", git_ref])
        return True
    except subprocess.CalledProcessError:
        c2cciutils.error(
            "gitattribute",
            "Error, see above",
        )
        return False


F_NULL = open(os.devnull, "w", encoding="utf-8")  # pylint: disable=consider-using-with


def eof(config: None, full_config: c2cciutils.configuration.Configuration, args: Namespace) -> bool:
    r"""
    Check the non empty text files end with "\n".

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config

    try:
        success = True

        sys.stdout.flush()
        sys.stderr.flush()
        for filename in subprocess.check_output(["git", "ls-files"]).decode().split("\n"):
            if os.path.isfile(filename):
                if (
                    subprocess.call(
                        f"git check-attr -a '{filename}' | grep ' text: set'",
                        shell=True,  # nosec
                        stdout=F_NULL,
                    )
                    == 0
                ):
                    size = os.stat(filename).st_size
                    if size != 0:
                        with open(filename, encoding="utf-8") as open_file:
                            open_file.seek(size - 1)
                            if ord(open_file.read()) != ord("\n"):
                                if not args.fix:
                                    with open(filename, "a", encoding="utf-8") as open_file_write:
                                        open_file_write.write("\n")
                                else:
                                    c2cciutils.error(
                                        "eof",
                                        f"No new line at end of '{filename}' file.",
                                        filename,
                                    )
                                    success = False

        return success
    except subprocess.CalledProcessError:
        c2cciutils.error(
            "eof",
            "Error, see above",
        )
        return False


def workflows(
    config: None,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Check each workflow have a timeout and/or do not use blacklisted images.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config, args

    success = True
    files = glob.glob(".github/workflows/*.yaml")
    files += glob.glob(".github/workflows/*.yml")
    for filename in files:
        with open(filename, encoding="utf-8") as open_file:
            workflow = yaml.load(open_file, Loader=yaml.SafeLoader)

        for name, job in workflow.get("jobs").items():
            if job.get("timeout-minutes") is None:
                c2cciutils.error(
                    "workflows",
                    f"The workflow '{filename}', job '{name}' has no timeout",
                    filename,
                )
                success = False

    return success


def black(
    config: c2cciutils.configuration.ChecksBlackConfig,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run black check on all files including Python files without .py extension.

    config is like:
      ignore_patterns_re: [] # list of regular expression we should ignore

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del full_config

    try:
        sys.stdout.flush()
        sys.stderr.flush()
        cmd = ["black"]
        if not args.fix:
            cmd += ["--color", "--diff", "--check"]
        cmd.append("--")
        python_files = c2cciutils.get_git_files_mime(ignore_patterns_re=config.get("ignore_patterns_re", []))
        cmd += python_files
        if len(python_files) > 0:
            subprocess.check_call(cmd)
        return True
    except subprocess.CalledProcessError:
        c2cciutils.error(
            "black",
            "Error, see above",
        )
        return False


def isort(
    config: c2cciutils.configuration.ChecksIsortConfig,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run isort check on all files including Python files without .py extension.

    config is like:
      ignore_patterns_re: [] # list of regular expression we should ignore

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del full_config

    try:
        sys.stdout.flush()
        sys.stderr.flush()
        cmd = ["isort"]
        if not args.fix:
            cmd += ["--check-only", "--diff"]
        cmd.append("--")
        python_files = c2cciutils.get_git_files_mime(ignore_patterns_re=config.get("ignore_patterns_re", []))
        cmd += python_files
        if len(python_files) > 0:
            subprocess.check_call(cmd)
        return True
    except subprocess.CalledProcessError:
        c2cciutils.error(
            "isort",
            "Error, see above",
        )
        return False


def codespell(
    config: c2cciutils.configuration.ChecksCodespellConfig,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run codespell check on all files.

    If there is an `spell-ignore-words.txt` file we consider it with ignore word

    config is like:
        ignore_re: [] # list of patterns to be ignored
        arguments: [] # codespell arguments

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """

    try:
        cmd = c2cciutils.get_codespell_command(full_config, args.fix)
        cmd.append("--")
        ignore_res = [
            re.compile(r)
            for r in config.get(
                "ignore_re", c2cciutils.configuration.CODESPELL_IGNORE_REGULAR_EXPRESSION_DEFAULT
            )
        ]
        for filename in subprocess.check_output(["git", "ls-files"]).decode().strip().split("\n"):
            if os.path.isfile(filename):
                include = True
                for ignore_re in ignore_res:
                    if ignore_re.match(filename):
                        include = False
                        continue
                if include:
                    cmd.append(filename)
        sys.stdout.flush()
        sys.stderr.flush()
        subprocess.check_call(cmd)
        return True
    except subprocess.CalledProcessError:
        c2cciutils.error(
            "codespell",
            "Error, see above",
        )
        return False


def prettier(
    config: c2cciutils.configuration.ChecksPrettierConfig,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run prettier check on all the supported files.

    config is like:
      # Currently empty

    Arguments:
        config: The config
        full_config: The full config
        args: The parsed command arguments
    """
    del config, full_config

    success = True

    with c2cciutils.prettier.Prettier() as prettier_lib:
        for filename in subprocess.check_output(["git", "ls-files"]).decode().strip().split("\n"):
            if os.path.isfile(filename):
                info = prettier_lib.get_info(filename)
                if info.get("info", {}).get("ignored", False):
                    continue
                if not info.get("info", {}).get("inferredParser"):
                    continue

                print(f"Checking {filename}")

                prettier_config = info["config"]
                if not prettier_config:
                    prettier_config = {}
                prettier_config["parser"] = info["info"]["inferredParser"]

                if args.fix:
                    if not prettier_lib.format(filename, prettier_config):
                        success = False
                else:
                    if not prettier_lib.check(filename, prettier_config):
                        success = False
    return success


def snyk(
    config: c2cciutils.configuration.ChecksSnykConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run Snyk check.
    """
    del full_config, args

    snyk_exec, env = c2cciutils.snyk_exec()
    command = [snyk_exec, "test"] + config.get(
        "arguments", c2cciutils.configuration.CHECKS_SNYK_ARGUMENTS_DEFAULT
    )

    print(f"Running Snyk: {' '.join(command)}")
    sys.stdout.flush()
    sys.stderr.flush()
    test_proc = subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
    print(f"Snyk exit code: {test_proc.returncode}")
    # For the moment we don't return an error
    # return test_proc.returncode == 0
    return True


def snyk_code(
    config: c2cciutils.configuration.ChecksSnykCodeConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run Snyk code check.
    """
    del full_config, args

    snyk_exec, env = c2cciutils.snyk_exec()
    command = [snyk_exec, "code", "test"] + config.get(
        "arguments", c2cciutils.configuration.CHECKS_SNYK_CODE_ARGUMENTS_DEFAULT
    )
    print(f"Running Snyk: {' '.join(command)}")
    sys.stdout.flush()
    sys.stderr.flush()
    test_proc = subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
    print(f"Snyk exit code: {test_proc.returncode}")
    # For the moment we don't return an error
    # return test_proc.returncode == 0
    return True


def snyk_iac(
    config: c2cciutils.configuration.ChecksSnykIacConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run Snyk iac check.
    """
    del full_config, args

    snyk_exec, env = c2cciutils.snyk_exec()
    command = [snyk_exec, "iac", "test"] + config.get(
        "arguments", c2cciutils.configuration.CHECKS_SNYK_IAC_ARGUMENTS_DEFAULT
    )
    print(f"Running Snyk: {' '.join(command)}")
    sys.stdout.flush()
    sys.stderr.flush()
    test_proc = subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
    print(f"Snyk exit code: {test_proc.returncode}")
    return test_proc.returncode == 0


def snyk_fix(
    config: c2cciutils.configuration.ChecksSnykFixConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Run Snyk fix.
    """
    del full_config, args

    snyk_exec, env = c2cciutils.snyk_exec()
    command = [snyk_exec, "fix"] + config.get(
        "arguments", c2cciutils.configuration.CHECKS_SNYK_FIX_ARGUMENTS_DEFAULT
    )
    print(f"Running Snyk: {' '.join(command)}")
    sys.stdout.flush()
    sys.stderr.flush()
    test_proc = subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
    print(f"Snyk exit code: {test_proc.returncode}")
    sys.stdout.flush()
    sys.stderr.flush()
    test_proc = subprocess.run(["git", "diff"])  # pylint: disable=subprocess-run-check
    return True


def print_versions(
    config: c2cciutils.configuration.PrintVersions,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Print some tools versions.

    Arguments:
        config: The check section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del full_config, args

    return c2cciutils.print_versions(config)



from package import Package
from reqPackage import ReqPackage


class DistPackage(Package):
    """Wrapper class for pkg_resources.Distribution instances
      :param obj: pkg_resources.Distribution to wrap over
      :param req: optional ReqPackage object to associate this
                  DistPackage with. This is useful for displaying the
                  tree in reverse
    """

    def __init__(self, obj, req=None):
        super(DistPackage, self).__init__(obj)
        self.version_spec = None
        self.req = req

    def as_requirement(self):
        """Return a ReqPackage representation of this DistPackage"""
        return ReqPackage(self._obj.as_requirement(), dist=self)

    def as_required_by(self, req):
        """Return a DistPackage instance associated to a requirement
        This association is necessary for displaying the tree in
        reverse.
        :param ReqPackage req: the requirement to associate with
        :returns: DistPackage instance
        """
        return self.__class__(self._obj, req)

    def as_dict(self):
        return {'key': self.key,
                'package_name': self.project_name,
                'installed_version': self.version}



import distutils.core
import re
import setuptools

def parse(setup_py_content):
    """Parse a setup.py file and extract the arguments passed to the setup method"""
    # Make the setup method return the arguments that are passed to it
    def _save_passed_args(**kwargs):
        return kwargs

    distutils.core.setup = _save_passed_args
    setuptools.setup = _save_passed_args
    setup_py_content = setup_py_content.replace("setup(", "passed_arguments = setup(")

    # Fetch the arguments that were passed to the setup.py
    exec(setup_py_content, globals())
    return globals()["passed_arguments"]


def parse_name_and_version(setup_py_content):
    """Extract the name and version from a setup.py file"""
    passed_arguments = parse(setup_py_content)
    return passed_arguments.get("name"), passed_arguments.get("version")


def get_provenance(setup_py_content):
    """Provenance for a setup.py file is the index of the line that contains `install_requires`"""
    for index, line in enumerate(setup_py_content.splitlines()):
        if re.search(r"(packages|install_requires)\s*=", line):
            return index + 1
    return -1


def parse_requirements(setup_py_content):
    """Extract the dependencies from a setup.py file"""
    passed_arguments = parse(setup_py_content)
    requirements = passed_arguments.get("install_requires", passed_arguments.get("packages", []))
    return "\n".join(requirements)




class Package(object):
    """Abstract class for wrappers around objects that pip returns.
    This class needs to be subclassed with implementations for
    `render_as_root` and `render_as_branch` methods.
    """

    def __init__(self, obj):
        self._obj = obj
        self.project_name = obj.project_name
        self.key = obj.key

    def render_as_root(self, frozen):
        return NotImplementedError

    def render_as_branch(self, frozen):
        return NotImplementedError

    def render(self, parent=None, frozen=False):
        if not parent:
            return self.render_as_root(frozen)
        else:
            return self.render_as_branch(frozen)

    def __getattr__(self, key):
        return getattr(self._obj, key)

    def __repr__(self):
        return '<{0}("{1}")>'.format(self.__class__.__name__, self.key)
